package service

import (
	"context"
	"testing"

	"git.perx.ru/perxis/perxis-go/pkg/extension"
	"github.com/stretchr/testify/require"
)

func TestExtension_Action(t *testing.T) {
	tests := []struct {
		name    string
		desc    *extension.ExtensionDescriptor
		in      *extension.ActionRequest
		router  extension.ActionRouter
		want    string
		wantErr bool
	}{
		{
			name: "Router with 1 named action",
			desc: &extension.ExtensionDescriptor{Extension: "test", Title: "Test Extension", Description: "desc", Version: "v.0.0.1"},
			in:   &extension.ActionRequest{Action: "grpc:///test/action", SpaceID: "sp", EnvID: "env", CollectionID: "coll"},
			want: "Called action",
			router: extension.Named("action", func(ctx context.Context, url *extension.ActionURL, req *extension.ActionRequest) (*extension.ActionResponse, error) {
				return &extension.ActionResponse{State: extension.ResponseDone, Msg: "Called action"}, nil
			}),
			wantErr: false,
		},
		{
			name: "Router with 2 named actions",
			desc: &extension.ExtensionDescriptor{Extension: "test", Title: "Test Extension", Description: "desc", Version: "v.0.0.1"},
			in:   &extension.ActionRequest{Action: "grpc:///test/action2", SpaceID: "sp", EnvID: "env", CollectionID: "coll"},
			want: "Called action2",
			router: extension.Chain(extension.Named("action", func(ctx context.Context, url *extension.ActionURL, req *extension.ActionRequest) (*extension.ActionResponse, error) {
				return &extension.ActionResponse{State: extension.ResponseDone}, nil
			}), extension.Named("action2", func(ctx context.Context, url *extension.ActionURL, req *extension.ActionRequest) (*extension.ActionResponse, error) {
				return &extension.ActionResponse{State: extension.ResponseDone, Msg: "Called action2"}, nil
			})),
			wantErr: false,
		},
		{
			name: "Error (no extension)",
			desc: &extension.ExtensionDescriptor{Extension: "test", Title: "Test Extension", Description: "desc", Version: "v.0.0.1"},
			in:   &extension.ActionRequest{Action: "grpc:///test2/action2", SpaceID: "sp", EnvID: "env", CollectionID: "coll"},
			router: extension.Chain(extension.Named("action", func(ctx context.Context, url *extension.ActionURL, req *extension.ActionRequest) (*extension.ActionResponse, error) {
				return &extension.ActionResponse{State: extension.ResponseDone}, nil
			}), extension.Named("action2", func(ctx context.Context, url *extension.ActionURL, req *extension.ActionRequest) (*extension.ActionResponse, error) {
				return &extension.ActionResponse{State: extension.ResponseParametersRequired}, nil
			})),
			wantErr: true,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			s := &Extension{
				desc:   tt.desc,
				router: tt.router,
			}
			got, err := s.Action(context.Background(), tt.in)
			if tt.wantErr {
				require.Error(t, err)
				return
			}
			require.Equal(t, tt.want, got.Msg)
		})
	}
}
