package zap

import (
	"testing"

	"github.com/stretchr/testify/require"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
	"go.uber.org/zap/zaptest/observer"
)

func TestFilterCore_With(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)
	core = WithFilters(core)

	field := zap.String("k", "v")
	err := core.With([]zapcore.Field{field}).Write(zapcore.Entry{Message: "msg"}, nil)
	require.NoError(t, err)

	entries := logs.All()
	require.Len(t, entries, 1)
	require.Len(t, entries[0].Context, 1)
	require.True(t, field.Equals(entries[0].Context[0]))
}

func TestFilterCore_Write(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)
	core = WithFilters(core, ContainsField(zap.Bool("check", true)))

	err := core.With([]zapcore.Field{zap.Bool("check", true)}).Write(zapcore.Entry{Message: "msg"}, nil)
	require.NoError(t, err)

	err = core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{zap.Bool("check", true)})
	require.NoError(t, err)

	err = core.Write(zapcore.Entry{Message: "msg"}, nil)
	require.NoError(t, err)

	require.Equal(t, 2, logs.Len())
}

func TestNotContainsField(t *testing.T) {
	core, logs := observer.New(zapcore.InfoLevel)
	core = WithFilters(core, Not(ContainsField(zap.Int("b", 2))))

	err := core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{
		zap.Int("a", 1),
		zap.Int("b", 2),
	})
	require.NoError(t, err)

	err = core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{
		zap.Int("a", 1),
		zap.Int("b", 3),
	})
	require.NoError(t, err)

	err = core.Write(zapcore.Entry{Message: "msg"}, []zapcore.Field{})
	require.NoError(t, err)

	require.Equal(t, 2, logs.Len())
}
